eureqa_colors = {
    "WHITE": "#fff",
    "GREEN": "#62B331",
    "ORANGE": "#ec7e27",
    "YELLOW": "#FFD61C",
    "RED": "#bb1c1a",
    "GOLD": "#edb619",
    "PURPLE": "#8D5E9A",
    "PRIMARY_BLUE": "#238CBC",
    "LINK_BLUE": "#069",
    "LIGHT_GREY": "#ddd",
    "GREY": "#A9A5A0",
    "MEDIUM_GREY": "#777",
    "DARK_GREY": "#494745",
    "ICON_GREY": "#494745",
    "DEFAULT_FONT": "#333333",
    "LIGHT_FONT": "#777",
    "BACKGROUND": "#f4f4f4",
    "PRIMARY_BUTTON": "#ec7e27",
    "SECONDARY_BUTTON": "#A9A5A0",
    "BOX_SHADOW": "0px 2px 3px 0px #ddd",
    "BORDER_LIGHT": "1px solid #ddd",
    "BORDER_MEDIUM": "1px solid #777",
    "BORDER_DOTTED": "1px dotted #777",
    "BORDER_HIGHLIGHT": "1px solid #238CBC",
    "TABLE_ROW_SELECTED": "1px double #238CBC"
}